#ifndef DialogUtils_H
#define DialogUtils_H


/* Copyright 1988,1990,1991,1992,1993,1996 Capilano Computing Systems Ltd.
   All rights reserved.
   
   Modifications:
   
   1/30/91	Changed InitDialogUtils to InitCDDiaUtl
   2/28/92	Added FilterSetPopUpItem
   6/22/92	Changed meanings of arguments to ShowProgressBox and
   		ShowProgressPercent
   9/8/92	Added "rootWD" argument to GetWDRef
   5/3/93 CD	Added GetLongNumber and PutLongNumber
   8/30/96 CD	Added UpdateProgressBox
*/

#include <Carbon/Carbon.h>


#include "CDSys.h"		/* Determines which system/compiler we're running on */

#ifdef __cplusplus
extern "C" {
#endif 

/* "InitDialogUtils" needs to be called to initialize this module
   if the main program is written in something other than Modula */

pQUALIFIER void InitCDDiaUtl(void);

/* Check if running on MacOS X */

pQUALIFIER Boolean CDRunningMacOSX(void);



/* Our SetPort , it uses default Graphics Handle */
/* These routines currently use Get/SetGWorld */

pQUALIFIER void SetGWorldCD(CGrafPtr port, GDHandle gdh);

pQUALIFIER void SetPortCD(GrafPtr port);

pQUALIFIER void GetPortCD(GrafPtr *port);



/* "TrapImplemented" returns true if the given trap number is
   available on the system we're running on.  "toolTrap" must
   be set to true for a ToolBox trap, false for an OS trap */

pQUALIFIER Boolean TrapImplemented(Boolean toolTrap, short trapNum);


/* "FitsScreen" returns true if the given rectangle (in global
   screen coordinates) is completely visible given all the screens
   attached to the machine we're running on */

pQUALIFIER Boolean FitsScreen(Rect wRect);


/* Put up the alert box with the given ID and set msg1 and msg2
   to the first two parameters (^0 and ^1) */

pQUALIFIER short DoAlert(short alertID, const char *msg1, const char *msg2);


/* Put up the alert box with the given ID and set the ^0 parameter
   to the "STR " resource whose ID is msg1 and the ^1 parameter
   to the string msg2 */

pQUALIFIER short DoResAlert(short alertID, short msg1, const char msg2[]);
pQUALIFIER Boolean FileErrorAlert(short error, short code,FSRefPtr theFSRef, char *fileNamePtr, CFURLRef theURLRef);

/* "ErrorAlert" puts up an error alert box if "error" is non-zero.
   Most common file errors are decoded into a text message, or
   else an error number is displayed.  Requires ALRT resource 257.
   The integer "code" is converted to hex and displayed in
   the alert box. */

pQUALIFIER Boolean ErrorAlert(short error, short code);


/* "GetFileErrStrID" returns the STR resource ID of the error
   message string to match the given error number.  This assumes
   access to the appropriate set of strings in the main program */

pQUALIFIER short GetFileErrStrID(short error);



/* "GetResString" reads a resource of type STR with the given ID
   and returns the string in "str".  An empty string will be returned
   in case of any error. */

pQUALIFIER void GetResString(short resID, char *str, short strSize);


/* General purpose routines for manipulating dialog box items. */

pQUALIFIER void SetItemValue(DialogPtr diaPtr, short itemNum, short hilite);

pQUALIFIER void HiliteItem(DialogPtr diaPtr, short itemNum, short hilite);

pQUALIFIER void HiliteButton(DialogPtr diaPtr, short itemNum);

pQUALIFIER void SetItemMinMax(DialogPtr diaPtr, short itemNum, short min, short max);

pQUALIFIER void SetItemText(DialogPtr diaPtr, short itemNum, char str[]);


/* "SetItemBlock" sets the given text item to the given arbitrary-length
   text block */

pQUALIFIER void SetItemBlock(	DialogPtr	diaPtr,
								short		itemNum,
								Ptr			textPtr,
								short		textLen
							);


/* "GetItemBlock" returns a pointer to and the length of the text
   in the given text item.  NOTE: The text data is NOT copied.
   This procedure locks the item handle and returns a pointer
   to the data.  The handle is left locked. */

pQUALIFIER void GetItemBlock(	DialogPtr	diaPtr,
								short		itemNum,
								Ptr			*textPtr,
								short		*textLen
							);



/* Select all the text for the given edit text item */

pQUALIFIER void SelTextItem(DialogPtr diaPtr, short itemNum);


/* "SetTextItemEnable" enables or disables the given text item */

pQUALIFIER void SetTextItemEnable(DialogPtr diaPtr, short itemNum, Boolean enable);


/* "TypedSetTextItemEnable" enables or disables the given text item
   and allows the item type to be set.  This is intended primarily to
   allow an item to be switched from an editText to a staticText. */

pQUALIFIER void TypedSetTextItemEnable(DialogPtr diaPtr, short itemNum, short itemType, Boolean enable);


/* Get the numeric value of the contents of an editText
item in a dialog box.   Returns 65535 if any error, zero
if item is empty */

pQUALIFIER unsigned short GetNumber(DialogPtr diaPtr, short itemNum);


/* Place the value specified in the editText item of a dialog box */

pQUALIFIER void PutNumber(DialogPtr diaPtr, short itemNum, short value);


/* Get the numeric value of the contents of an editText
   item in a dialog box.   Returns 4294967295 if any error, zero
   if item is empty */

pQUALIFIER unsigned long GetLongNumber(DialogPtr diaPtr, short itemNum);


/* Place the value specified in the editText item of a dialog box */

pQUALIFIER void PutLongNumber(DialogPtr diaPtr, short itemNum, long value);
//
// Passed in a buffer of text, pass back a buffer with linefeeds remove
// Caller only deallocates buffer of returned text if lineFeed was found
//  

Boolean LineFeedFoundAndRemoved(char * inputText, Size textLength, char ** textWithLFRemoved, Size *newLength );

pQUALIFIER short GetItemValue(DialogPtr diaPtr, short itemNum);

pQUALIFIER short GetItemHeight(DialogPtr diaPtr, short itemNum);


/* Get text from an edit text box in a dialog */

pQUALIFIER void GetItemText(DialogPtr diaPtr, short itemNum, char *str, short strSize);


/* Returns true if the given control item is hilited active, false otherwise */

pQUALIFIER Boolean ActiveItem(DialogPtr diaPtr, short itemNum);



/* Make the given item invisible */

pQUALIFIER void HideItem(DialogPtr diaPtr, short itemNum);



/* Display a non-modal dialog as a progress status box using the given
   resID for the string to use.  If a progress box is already showing
   then only the string is changed.
   
   If "stringID" is 0 or the resource is not found, no string is displayed.
   
   This call uses "ParamText" to display the given strings.  The resource
   string is substituted for "^0" and "varStr" is substituted for "^1".
*/

pQUALIFIER void ShowProgressBox(short dialogID, short stringID, const char varStr[]);


/* "UpdateProgressBox" redraws the current progress box (if any)
   with the same contents specified in previous calls to
   ShowProgressBox and ShowProgressPercent */

pQUALIFIER void UpdateProgressBox();



/* "ShowProgressPercent" displays a percentage equal to
   (current * 100) DIV outOf after the strings put up by a previous
   call to "ShowProgressBox".  A subsequent call to "ShowProgressBox"
   will erase the percentage.  "percentStr" is a string containing
   text to put immediately after the number, normally just "%".
   
   If "outOf" is <= 0, then the value "current" is displayed verbatim
   without conversion to a percentage.  It must be in the range 0 <= x <= 32767.
   If current < outOf, the value "0" is displayed.  If current > outOf
   (except for the case outOf <= 0 described above), the value "100" is displayed.
   
   The percentage string consisting of the number plus the "percentStr"
   is substituted for dialog item number 2, which should be a static text item.
*/

pQUALIFIER void ShowProgressPercent(long current, long outOf,const char percentStr[]);


/* "CloseProgressBox" closes any currently open progress box */

pQUALIFIER void CloseProgressBox(void);


/* The following routines replace GetNewDialog, DisposDialog and ModalDialog
   for dialogs that want to use Enter for OK and implement Cut Copy and Paste
   for any text boxes.  "returnItemA" and "returnItemB" are the item numbers
   of any items in which a carriage return is allowed in text, or zero.
   In any other items CR is treated like a tab. */

pQUALIFIER DialogPtr TextGetNewDialog(short dialogID, short returnItemA, short returnItemB);


pQUALIFIER void TextDisposDialog(DialogPtr diaPtr);

pQUALIFIER void TextModalDialog(short *itemHit);



/* The following set of routines supercedes the above and is used to set
   up a general dialog box with a filter procedure to handle:
   
   - any number of ListManager lists
   - any number of text items which accept carriage returns
   - closing using the Enter key
   - return on timeout
*/

/* "FilterGetNewDialog" is called instead of "GetNewDialog".  "dialogID"
   is the resource ID of the desired dialog.  "okItem" is the item
   number to return when the Enter key or carriage return (if enabled)
   is pressed */

pQUALIFIER DialogPtr FilterGetNewDialog(	short	dialogID, short	okItem);



/* "FilterSetPopUpItem" sets the given item to be a heading for a pop-up
   menu item.  If "title" is an empty string, then the existing item
   is disabled and erased from the screen. */

pQUALIFIER void FilterSetPopUpItem( short	itemNum, Str255	title );


/* "FilterSetPopUpStatus" sets the visual status of the given pop-up item. */

pQUALIFIER void FilterSetPopUpStatus( short	itemNum, Boolean	enabled,  Boolean	italic);


/* "FilterHilitePopUpItem" visually highlights (active == TRUE)
   or dehighlights (active == FALSE) the given popup menu item. */

pQUALIFIER void FilterHilitePopUpItem(short	itemNum, Boolean	active);


/* "FilterAddCRItem" adds the given item to the list of text boxes
   that can accept carriage return input.  If the given item is
   active, a CR will be placed in the string and will not be
   taken as a close. */

pQUALIFIER void FilterAddCRItem(	short	itemNum);


/* "FilterRemoveCRItem" removes the given item to the list of text boxes
   that can accept carriage return input.   */

pQUALIFIER void FilterRemoveCRItem(	short	itemNum);


/* "FilterSetCRItemEnable" allows you to dynamically enable or disable
   the entry of carriage returns into to given text item.  It must
   already have been set up with "FilterAddCRItem" above.
   If the item is disabled, pressing a CR will cause a beep
   and will otherwise be ignored. */

pQUALIFIER void FilterSetCRItemEnable(	short itemNum, Boolean	enable );



/* "FilterAddListItem" provides the information necessary for the
   filter procedure to handle a ListManager list.
   "listRect" is the hit rectangle for the list, "listItem" is
   the item number to return when the list is hit,
   "doubleClickItem" is the item number to return when a
   double-click is detected in the list.  Use "FilterSetListItem"
   to set the list handle after it is created.  The handle
   is set separately since it may change frequently as the
   list is updated. */

pQUALIFIER void FilterAddListItem(	Rect listRect, short listItem, short doubleClickItem);



/* "FilterSetListItem" sets the list handle for the given list item */

pQUALIFIER void FilterSetListItem(	short itemNum, ListHandle	hList);


/* "FilterSetTimeout" enables timeout return, i.e. when the system
   time (i.e. the time in a ToolBox event) exceeds the given value,
   "FilterModalDialog" will return the given item number.  This
   routine must be re-called each time this return happens to reset
   the time. */

pQUALIFIER void FilterSetTimeout(	long time, short itemNum);


/* "FilterModalDialog" waits until an item is hit (or timeout occurs,
   if enabled) then returns the number of the hit item. */

pQUALIFIER void FilterModalDialog(	short	*itemHit);



/* "FilterDisposDialog" disposes of the dialog and internal structures
   created for the filter procedure.
   
   IMPORTANT: This routine DOES NOT dispose of the list handles.  These
    	      must be disposed of by the caller BEFORE calling this
	      because the ListManager refers to the window structure
	      in its dispose routine.
*/

pQUALIFIER short FilterDisposDialog(	DialogPtr	diaPtr);








/* SetMenuItem accesses the menu structure directly, rather than using
   toolbox routine "SetItem" because "SetItem" interprets some characters
   as having special meanings, contrary to the documentation */

pQUALIFIER void SetMenuItem(MenuRef menu, short item, char itemText[]);


pQUALIFIER void CDDiaUtl_Init(void);


pQUALIFIER void UpdateMenuBox(short itemNum, MenuHandle menu, short current);
#ifdef __cplusplus
}
#endif

#endif /*DialogUtils_H*/

